<?php 

class DonorCom_Admin  {
	
	var $hook 		= 'donor-com';
	var $longname	= 'donor.com Configuration';
	var $shortname	= 'donor.com';
	var $homepage	= 'http://yoast.com/wordpress/donor-com/';
	var $optionname = 'donorcom';
	var $accesslvl	= 'manage_options';
	
	function DonorCom_Admin() {
		add_action( 'admin_init', array(&$this, 'option_init') );
		
		add_action( 'admin_menu', array(&$this, 'register_settings_page') );
		add_filter( 'plugin_action_links', array(&$this, 'add_action_link'), 10, 2 );
		
		add_action( 'admin_print_scripts', array(&$this,'config_page_scripts'));
		add_action( 'admin_print_styles', array(&$this,'config_page_styles'));	
		
		add_action( 'wp_dashboard_setup', array(&$this,'widget_setup'));	

		register_activation_hook( DONORCOM_FILE, array(&$this, 'activate') );
		
		// This would delete the settings when the plugin is deactivated, which is not useful
		// The uninstall.php in the directory would delete the settings when the plugin is deleted, which *is* useful.
		// This hook is still here for debug purposes though.
		// register_deactivation_hook( DONORCOM_FILE, array(&$this, 'deactivate') );
	}

	function deactivate() {
		delete_option('donorcom');
	}
	
	// This function runs on plugin activation
	function activate() {
		// Set default options
		$options = array(
			'apicode' => '',
			'apiport' => 443,
			'cachelocal' => 'on',
			'cacheserver' => 'on',
		);
		add_option('donorcom', $options);
	}
		
	function option_init() {
		register_setting( 'donorcom_options', $this->optionname, array(&$this,'option_sanitization') );
	}
	
	function register_settings_page() {
		add_options_page($this->longname, $this->shortname, $this->accesslvl, $this->hook, array(&$this,'config_page'));
	}

	function plugin_options_url() {
		return admin_url( 'options-general.php?page='.$this->hook );
	}

	/**
	 * Add a link to the settings page to the plugins list
	 */
	function add_action_link( $links, $file ) {
		if ( $file == DONORCOM_FILE ) {
			$settings_link = '<a href="' . $this->plugin_options_url() . '">' . __('Settings') . '</a>';
			array_unshift( $links, $settings_link );
		}
		return $links;
	}
		
	function option_sanitization( $input ) {
		// Make sure the API code is valid
		preg_match( '/^([a-zA-Z0-9]{2,8})/', trim( $input['apicode'] ), $matches );		
		if ( isset( $matches[1] ) )
			$input['apicode'] = $matches[1];
		else
			$input['apicode'] = '';
		
		// Make sure the port isn't invalid
		$input['apiport'] = intval($input['apiport']);
		if ( $input['apiport'] == '' || $input['apiport'] == 0 )
			$input['apiport'] = 443;
		
		// Make sure the stored Page ID's are indeed integers and nothing else
		foreach ( array('store','donate','sponsor') as $type ) {
			if ( isset( $input[ $type.'url' ] ) )
				$input[ $type.'url' ] = (int) $input[ $type.'url' ];
		}
		
		return $input;
	}
	
	function config_page_styles() {
		if (isset($_GET['page']) && $_GET['page'] == $this->hook) {
			wp_enqueue_style('dashboard');
			wp_enqueue_style('thickbox');
			wp_enqueue_style('global');
			wp_enqueue_style('wp-admin');
			wp_enqueue_style('donorcom-admin-css', plugin_dir_url( __FILE__ ) .'donor_plugin.css');
		}
	}
	
	function config_page_scripts() {
		if (isset($_GET['page']) && $_GET['page'] == $this->hook) {
			wp_enqueue_script('jquery');
			wp_enqueue_script('postbox');
			wp_enqueue_script('dashboard');
			wp_enqueue_script('thickbox');
		}
	}

	/**
	 * Create a Checkbox input field
	 */
	function checkbox($id, $label, $label_left = false ) {
		$options = get_option($this->optionname);
		$option = $this->optionname;
		
		$output_label = '<label for="'.$id.'">'.$label.'</label>';
		
		if ( !isset( $options[$id] ) )
			$options[$id] = false;
			
		$output_input = '<input class="checkbox" type="checkbox" id="'.$id.'" name="'.$option.'['.$id.']"'. checked($options[$id],'on',false).'/> ';
		
		if( $label_left ) {
			$output = $output_label . $output_input;
		} else {
			$output = $output_input . $output_label;
		}
		return $output . '<div class="clear"></div>';
	}
	
	/**
	 * Create a Text input field
	 */
	function textinput($id, $label) {
		$options = get_option($this->optionname);
		$option = $this->optionname;
		
		$val = esc_attr($options[$id]);
		
		return '<label class="textinput" for="'.$id.'">'.$label.':</label><input class="textinput" type="text" id="'.$id.'" name="'.$option.'['.$id.']" value="'.$val.'"/>';
	}

	/**
	 * Create a postbox widget
	 */
	function postbox( $id, $title, $content ) {
	?>
		<div id="<?php echo $id; ?>" class="postbox">
			<div class="handlediv" title="Click to toggle"><br /></div>
			<h3 class="hndle"><span><?php echo $title; ?></span></h3>
			<div class="inside">
				<?php 
					echo $content; 
					echo '<div class="clear"></div>';
				?>
			</div>
		</div>
	<?php
	}	

	/**
	 * Easily create pages
	 */
	function create_page( $title, $content, $type ){

		// check if the Donor.com page you're trying to create already exists
		$results = get_pages( array('meta_key' => '_generator', 'meta_value' => 'donorcom-plugin') );
		if( !empty( $results ) ) {
			foreach( $results as $res ) {
				$meta_value = get_post_meta($res->ID, '_pagetype', true);
				// If the specific page type already exists, just return that ID instead of creating a new page.
				if( $meta_value == $type ) {
					return $res->ID;
				}
			}
		}

		// If we haven't returned false yet, the page doesn't exist, so let's create it
		$page_id = wp_insert_post(
			array(
				'comment_status' 	=> 'closed',
				'post_content'		=> trim($content)."\n",
				'post_status'		=> 'publish',
				'post_title'		=> $title,
				'post_type'			=> 'page'
			)
		);

		// create custom field in order to be able to check later
		add_post_meta($page_id, '_generator', 'donorcom-plugin');
		add_post_meta($page_id, '_pagetype', $type);

		return $page_id;
	}
		
	function config_page() {
		$options = get_option($this->optionname);
		
		foreach ( array( 'store', 'donate', 'sponsor', 'account') as $key ) {
			if ( isset($_GET['create_'.$key.'_page']) && $_GET['create_'.$key.'_page'] == 1 ) {
				$ret = $this->create_page( ucfirst($key), '[donor type="'.$key.'"]', $key );
				if ( $ret ) {
					$options[ $key.'url' ] = $ret;
					update_option($this->optionname, $options);
					echo '<div class="updated" id="message">';
					echo '<p><strong>';
					echo '<a href="'.get_permalink( $ret ).'">'.ucfirst( $key) .' '.__('Page').'</a>'.__(' successfully created.');
					echo '</strong></p>';
					echo '</div>';
				} 
			}	
		}
		
		if ( isset($_GET['settings-updated']) && $_GET['settings-updated'] == true ) {
			echo '<div class="updated" id="message"><p><strong>'.__('Settings Updated.').'</strong></p></div>';
		}
		
		?>
		<div class="wrap">
			<a href="http://donor.com/"><div id="yoast-icon" style="background: url(<?php echo plugin_dir_url( __FILE__ ); ?>img/donor_icon_32.png) no-repeat;" class="icon32"><br /></div></a>
			<h2>donor.com options</h2>
			<div class="postbox-container" style="width:70%;">
				<div class="metabox-holder">	
					<div class="meta-box-sortables">
						<form action="options.php" method="post" id="donorcom-conf" enctype="multipart/form-data">
							<?php
							if ( function_exists('wp_nonce_field') )
								wp_nonce_field('donorcom-config');

							settings_fields('donorcom_options');

							/**
							 * Required Settings Box
							 */

							$content = '<p>'.__('If you don\'t enter the following, the plugin will not work:').'</p>';
							$content .= $this->textinput( 'apicode', __('donor.com Client Code') );
							$content .= '<br class="clear"><p>'.make_clickable( __('If you don\'t have a donor.com account, please visit http://donor.com/signup for a free 30-day trial or visit http://donor.com/tv to find out how donor.com can help your non-profit organization with an integrated fundraising solution.') ).'</p>';
							$this->postbox('required','Required Settings', $content );

							/**
							 * Pages Box
							 */

							$content = '<p>'.__('Please specify the page you\'d like to use, or click the button to create one:').'</p>';
							foreach ( array( 'store' => 'Store', 'sponsor' => 'Sponsor', 'donate' => 'Donate', 'account' => 'Account') as $type => $label ) {
								if ( !isset($options[$type.'url']) )
									$options[$type.'url'] = 0;
								$content .= '<label class="select" for="'.$type.'url">'.$label.' '.__('Page').':</label>';
								$content .= wp_dropdown_pages(
									array(
										'echo' => 0, 
										'name' => $this->optionname.'['.$type.'url]', 
										'selected' => $options[$type.'url'], 
										'show_option_none' => __('Select a page'),
										'option_none_value' => 0
									)
								);
								if ( !isset( $options[$type.'url'] ) || $options[$type.'url'] == 0 ) {
									$content .= '<a class="button" href="'.admin_url('options-general.php?page=donor-com').'&amp;create_'.$type.'_page=1">'.__('Create').' '.$label.' '.__('page').'</a>';
								} else {
									$content .= '<a class="button" href="'.get_edit_post_link($options[$type.'url']).'">'.__('Edit').' '.$label.' '.__('page').'</a>';
								}									
							}
							
							$this->postbox('urls','Select or Create Pages', $content );

							/**
							 * Shortcode Explanation Box
							 */

							$content = '<p>'.__('You can use the donor.com shortcode in your posts and pages. Use it like this:').'</p>';
							$content .= '<pre>[donor type="donate"]</pre>';
							$content .= '<p>'.__('Accepted types are donate, sponsor, store and account. If you want to show a cart, you can add an extra variable "mode", like so:').'</p>';
							$content .= '<pre>[donor type="store" mode="cart"]</pre>';
							$content .= '<p>'.__('That will show a cart instead of the default canvas.').'</p>';
							$this->postbox('shortcode-help', __('Shortcode Help'), $content);

							/**
							 * Advanced Settings & Debug Box
							 */
																						
							$content = $this->checkbox( 'disablecss', __('Disable CSS?') );
							$content .= '<p class="expl">'.__('Only do this when you know how to style the donor.com output yourself.').'</p>';
							$content .= '<h4>'.__('Debug').'</h4>';
							$content .= '<p>'.__('Use these options only when talking to donor.com support agents').'</p>';
							$content .= $this->textinput( 'apiport', __('API Port') );
							$content .= $this->checkbox( 'cachelocal', __('Allow local caching') );
							$content .= $this->checkbox( 'cacheserver', __('Allow server caching') );

							$this->postbox('advanced','Advanced Settings', $content );


							?>
							<div class="submit">
								<input type="submit" class="button-primary" name="submit" value="Update Your donor.com Settings &raquo;" />
							</div>
						</form>
					</div>
				</div>
			</div>
			<div class="postbox-container" style="width:20%;">
				<div class="metabox-holder">	
					<div class="meta-box-sortables">
						<?php
							$this->plugin_like('donor-com');
							$this->plugin_support('donor-com');
							$this->news(); 
						?>
					</div>
					<br/><br/><br/>
				</div>
			</div>
		</div>
<?php		
	}
		
	/**
	 * Create a "plugin like" box.
	 */
	function plugin_like() {
		$content = '<p>'.__('Why not do any or all of the following:','ystplugin').'</p>';
		$content .= '<ul>';
		$content .= '<li><a href="'.$this->homepage.'">'.__('Link to it so other folks can find out about it.','ystplugin').'</a></li>';
		$content .= '<li><a href="http://wordpress.org/extend/plugins/'.$this->hook.'/">'.__('Give it a good rating on WordPress.org.','ystplugin').'</a></li>';
		$content .= '</ul>';
		$this->postbox($this->hook.'like', 'Like this plugin?', $content);
	}	
	
	/**
	 * Info box with link to the support forums.
	 */
	function plugin_support() {
		$content = '<p>'.__('If you have any problems with this plugin or good ideas for improvements or new features, please talk about them in the','ystplugin').' <a href="http://wordpress.org/tags/'.$this->hook.'">'.__("Support forums",'ystplugin').'</a>.</p>';
		$content .= '<p>'.make_clickable(__('You can also visit http://donor.com/support.')).'</p>';
		$this->postbox($this->hook.'support', 'Need support?', $content);
	}

	/**
	 * Box with latest news from donor.com
	 * Make sure the var feed above is filled so this box shows up
	 */
	function news() {
		if ( empty($this->feed) )
			return;
			
		require_once(ABSPATH.WPINC.'/rss.php');  
		if ( $rss = fetch_rss( $this->feed ) ) {
			$content = '<ul>';
			$rss->items = array_slice( $rss->items, 0, 3 );
			foreach ( (array) $rss->items as $item ) {
				$content .= '<li class="yoast">';
				$content .= '<a class="rsswidget" href="'.clean_url( $item['link'], $protocolls=null, 'display' ).'">'. htmlentities($item['title']) .'</a> ';
				$content .= '</li>';
			}
			$content .= '<li class="rss"><a href="'.$this->feed.'">Subscribe with RSS</a></li>';
			$this->postbox('donorlatest', 'Latest news from donor.com', $content);
		} else {
			$this->postbox('donorlatest', 'Latest news from donor.com', 'Nothing to say...');
		}
	}

	function text_limit( $text, $limit, $finish = ' [&hellip;]') {
		if( strlen( $text ) > $limit ) {
	    	$text = substr( $text, 0, $limit );
			$text = substr( $text, 0, - ( strlen( strrchr( $text,' ') ) ) );
			$text .= $finish;
		}
		return $text;
	}

	function db_widget() {
		if ( empty($this->feed) )
			return;

		$options = get_option('donorcomdbwidget');
		if (isset($_POST['donorcom_removedbwidget'])) {
			$options['removedbwidget'] = true;
			update_option('donorcomdbwidget',$options);
		}			
		if ($options['removedbwidget']) {
			echo "If you reload, this widget will be gone and never appear again, unless you decide to delete the database option 'donorcomdbwidget'.";
			return;
		}
		require_once(ABSPATH.WPINC.'/rss.php');
		if ( $rss = fetch_rss( $this->feed ) ) {
			echo '<div class="rss-widget">';
			echo '<a href="http://donor.com/" title="Go to donor.com"><img src="'.plugin_dir_url( __FILE__ ).'img/donor_icon_32.png" class="alignright" alt="donor.com"/></a>';			
			echo '<ul>';
			$rss->items = array_slice( $rss->items, 0, 3 );
			foreach ( (array) $rss->items as $item ) {
				echo '<li>';
				echo '<a class="rsswidget" href="'.clean_url( $item['link'], $protocolls=null, 'display' ).'">'. htmlentities($item['title']) .'</a> ';
				echo '<span class="rss-date">'. date('F j, Y', strtotime($item['pubdate'])) .'</span>';
				echo '<div class="rssSummary">'. $this->text_limit($item['summary'],250) .'</div>';
				echo '</li>';
			}
			echo '</ul>';
			echo '<div style="border-top: 1px solid #ddd; padding-top: 10px; text-align:center;">';
			echo '<a href="'.$this->feed.'"><img src="'.get_bloginfo('wpurl').'/wp-includes/images/rss.png" alt=""/> Subscribe with RSS</a>';
			echo '<form class="alignright" method="post"><input type="hidden" name="donorcom_removedbwidget" value="true"/><input title="Remove this widget from all users dashboards" type="submit" value="X"/></form>';
			echo '</div>';
			echo '</div>';
		}
	}

	function widget_setup() {
		$options = get_option('donorcomdbwidget');
		if (!$options['removedbwidget'])
	    	wp_add_dashboard_widget( 'donorcom_db_widget' , 'The Latest news from donor.com' , array(&$this, 'db_widget'));
	}		
}
$dsa = new DonorCom_Admin();
