<?php
/*
Plugin Name: donor.com fundraising gadget
Version: 1.0.1
Plugin URI: 
Description: Integrate your donor.com fundraising system in your WordPress to allow your donors to donate, buy product, sponsor children and manage their accounts.
Author: Joost de Valk - Yoast
Author URI: http://yoast.com/
*/

class DonorCom {
	
	function DonorCom() {
		
		// Loading of scripts and styles only needs to happen on the frontend.
		if ( !is_admin() ) {
			// Add the necessary scripts and styles
			add_action( 'wp_print_styles', array(&$this, 'load_style'), 90 );
			add_action( 'wp_print_scripts', array(&$this, 'load_scripts'), 90 );
			add_action( 'wp_footer', array(&$this, 'print_script'), 90 );
		} else {
			define('DONORCOM_FILE', plugin_basename( __FILE__ ) );

			// Load admin scripts
			require_once 'donor-admin.php';
		}
		
		// Instantiate Widget
		// Needs to be instantiated in backend too to allow usage.
		add_action('widgets_init', create_function('', 'return register_widget("DonorComWidget");'));
		
		// Instantiate the shortcode
		add_shortcode( 'donor', array(&$this, 'shortcode') );		
	}
	
	// Determine whether current URL is opened using HTTPS or HTTP and return transport protocol 
	// accordingly to allow proper embedding of scripts and styles
	function http() {
		if ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on' )
			return 'https';
		return 'http';
	}
	
	// Return cloudfront URL
	function host() {
		return $this->http().'://api.donor.com/gadgets/';
	}
	
	function load_style() {
		$options = get_option('donorcom');
		
		if ( !isset($options['disablecss']) || !$options['disablecss'] ) {
			$host = $this->host();
			wp_register_style( 'donorcom_style', $host.'css/donor-css.css', array(), false, 'screen' );
			wp_enqueue_style( 'donorcom_style' );
		}
	}

	function load_scripts() {
		// Grab the cloudfront host
		$host = $this->host();
		
		// Register the different scripts, see http://codex.wordpress.org/Function_Reference/wp_register_script
		wp_register_script( 'donor-jquery-ui', $host.'js/jquery-ui.js', array('jquery'), false, true );
		wp_register_script( 'pure', $host.'js/pure.js', array('jquery', 'jquery-ui-core'), false, true );
		wp_register_script( 'chained-select', $host.'js/chainedselects.js', array('jquery', 'jquery-ui-core'), false, true );
		wp_register_script( 'jquery-cookie', $host.'js/jquery.cookie.js', array('jquery'), false, true );
		wp_register_script( 'jquery-store', $host.'js/jquery.store.js', array('jquery'), false, true );
		wp_register_script( 'dwapi', $host.'js/dwapi.js', array('jquery'), false, true );
		wp_register_script( 'donorcom', $host.'js/donor.com.js', array('jquery'), false, true );

		// Enqueue all necessary scripts, http://codex.wordpress.org/Function_Reference/wp_enqueue_script
		// First three scripts do not need to be registered as they're shipped with WordPress core
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'donor-jquery-ui' );
		wp_enqueue_script( 'json2' );

		wp_enqueue_script( 'chained-select' );
		wp_enqueue_script( 'jquery-cookie' );
		wp_enqueue_script( 'jquery-store' );
		wp_enqueue_script( 'dwapi' );
		wp_enqueue_script( 'donorcom' );
	}
	
	function print_script() {
		
		$options = get_option('donorcom');
		
		// If we don't have an API code, bail, otherwise the scripts will error anyway.
		if ( !isset( $options['apicode'] ) || trim( $options['apicode'] ) == '' )
			return;
		
		if ( empty( $options['apiport'] ) || $options['apiport'] == 0 )
			$options['apiport'] = 443;
		?>
<script type="text/javascript">
	DW.api.dwcode = '<?php echo $options['apicode']; ?>';
	DW.api.port = '<?php echo $options['apiport']; ?>'; 
	DW.api.run({
		clear: '.donor-gadget-loading',
<?php 
					if ( isset( $options['storeurl'] ) && $options['storeurl'] != 0 ) {
						echo "\t\tcheckout_url: '". get_permalink( $options['storeurl'] ) ."',\n";
						echo "\t\tcatalog_url: '". get_permalink( $options['storeurl'] ) ."',\n";
					}
					if ( isset( $options['sponsorurl'] ) && $options['sponsorurl'] != 0 ) {
						echo "\t\tsponsor_list_url: '". get_permalink( $options['sponsorurl'] ) ."',\n";
						echo "\t\tsponsor_checkout_url: '". get_permalink( $options['sponsorurl'] ) ."',\n";
					}
					if ( isset( $options['donateurl'] ) && $options['donateurl'] != 0 ) {
						echo "\t\tdonate_list_url: '". get_permalink( $options['donateurl'] ) ."',\n";
						echo "\t\tdonate_checkout_url: '". get_permalink( $options['donateurl'] ) ."',\n";
					}
?>
		cacheLocal: <?php echo ( $options['cachelocal'] == 'on' ) ? 'true' : 'false'; ?>,
		cacheServer: <?php echo ( $options['cachelocal'] == 'on' ) ? 'true' : 'false'; ?> 
	});
</script>
		<?php
	}
	
	// Creates standard WordPress Shortcode, see http://codex.wordpress.org/Shortcode_API
	function shortcode( $atts, $content = '' ) {
		$options = get_option('donorcom');
		if ( !isset( $options['apicode'] ) || trim( $options['apicode'] ) == '' ) {
			if ( current_user_can('manage_options') )
				return '<p>'.__('Make sure to enter the donor.com settings on the config page').'</p>';
			else
				return '';
		}
			
		
		// Extract shortcode, default to donate widget.
		extract( shortcode_atts( array(
			'type' => 'donate',
			'mode' => '',
		), $atts ) );
		
		// Check whether type is a valid value; if not, return an error.
		$type = trim( strtolower( $type ) );
		if ( ! in_array( $type, array('donate', 'sponsor', 'store', 'account', 'loading') ) )
			return '<p>Unrecognized Donor.com shortcode type.</p>';
		
		// Check whether mode is a valid value; if not, return an error.
		$mode = trim( strtolower( $mode ) );
		if ( empty( $mode ) )
			$mode = 'canvas';
		else if ( ! in_array( $mode, array('cart', 'canvas', 'featured') ) )
			return '<p>Unrecognized Donor.com shortcode mode.</p>';

		// Return the proper content
		if ( $type == 'loading' )
			return '<div class="donor-gadget-loading">'.$content.'</div>';
		else	
			return '<div class="donor-gadget-loading"></div><div id="donor-'.$type.'-'.$mode.'" class="donor-gadget"></div>';
	}
}
$donorcom = new DonorCom();

/**
 * Donor.com Widget Class
 */
class DonorComWidget extends WP_Widget {
    /** constructor */
    function DonorComWidget() {
        parent::WP_Widget(false, $name = 'Donor.com Widget');	
    }

    /** @see WP_Widget::widget */
    function widget($args, $instance) {		
        extract( $args );
        $title = apply_filters('widget_title', $instance['title']);

		echo $before_widget; 
		
		if ( $title )
        	echo $before_title . $title . $after_title; 

		if ( empty($instance['type']) )
			$instance['type'] = 'donate-canvas';
		
		echo '<div class="donor-gadget-loading"></div>';
		echo '<div id="donor-'.$instance['type'].'" class="donor-gadget"></div>';

		echo $after_widget; 
    }

    /** @see WP_Widget::update */
    function update($new_instance, $old_instance) {				
		$instance = $old_instance;
		$instance['title'] 	= strip_tags($new_instance['title']);
		$instance['type'] 	= $new_instance['type'];
        return $instance;
    }

    /** @see WP_Widget::form */
    function form($instance) {				
        $title = esc_attr( $instance['title'] );
        ?>
         <p>
          <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label> 
          <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" />
        </p>
         <p>
          	<label for="<?php echo $this->get_field_id('type'); ?>"><?php _e('Widget:'); ?></label> 
			<select id="<?php echo $this->get_field_id('type'); ?>" name="<?php echo $this->get_field_name('type'); ?>">
				<?php
					// types are used directly in generating the <div>.
					$types = array(
						'donate-canvas'	=> 'Donate: Canvas',
						'donate-cart' 	=> 'Donate: Cart',
						'store-canvas'	=> 'Store: Canvas',
						'store-cart' 	=> 'Store: Cart',
						'sponsor-canvas'=> 'Sponsor: Canvas',
						'sponsor-cart'	=> 'Sponsor: Cart',
						'account-canvas'=> 'Account: Canvas',
					);
					foreach ($types as $type => $desc) {
						$sel = '';
						if ( $type == $instance['type'] )
							$sel = 'selected="selected" ';
						echo '<option '.$sel.'value="'.$type.'">'.$desc.'</option>';
					}
				?>
			</select>
        </p>
        <?php 
    }

}
